import git
import owners
import integrations


def pull_request_ready_to_merge(github_token, org, repo, pr_number, owners_file='OWNERS'):
    """
    Using the GitHub API, check whether a pull request is ready to be merged. Adds a comment to the
    pull request that tags anyone who owns a file in the diff.
    :param github_token: A GitHub API token, used to read information and add a comment.
    :param org: A string name of the GitHub organization that owns the repository.
    :param repo: A string name of the GitHub repository for this pull request.
    :param pr_number: An integer ID for the GitHub pull request.
    :param owners_file: A relative path inside the repository where the OWNERS file is defined.
    :return: A boolean that represents whether the pull request can be merged.
    """
    github_repo = git.GitHub(github_token=github_token, org_name=org, repo_name=repo)
    pull_request = github_repo.get_pull_request(pr_number=pr_number)
    owner_lines = github_repo.read_file_lines(file_path=owners_file)
    owner_ids_and_globs = owners.parse(owner_lines)
    reviewers, required = owners.get_owners_of_files(owner_ids_and_globs, pull_request.files)
    individual_reviewers = github_repo.expand_teams(reviewers, except_login=pull_request.author)
    # individual_reviewers.append(pull_request.get_reviewers(owners_lines=['foo *.js', ]))
    if individual_reviewers:
        pull_request.assign_to(individual_reviewers[0])
        comment = pull_request.generate_comment(reviewers=individual_reviewers, required=required)
        pull_request.create_or_update_comment(comment)
    if required:
        return pull_request.all_have_signed_off(required)
    return pull_request.one_has_signed_off(individual_reviewers)


__all__ = [
    'pull_request_ready_to_merge',
    'GitHub',
    'integrations',
]



import mock_github
from base import MockPyGithubTests
from lgtm import git


class GitTests(MockPyGithubTests):

    def test_get_team_members(self):
        mock_github.create_fake_org(teams=[mock_github.MockTeam('team1', ['bat', 'baz'])])
        git_hub = git.GitHub('foo', 'bar', 'bat')
        self.assertEquals(git_hub.get_team_members('OrgName/team1'), ['bat', 'baz'])

    def test_expand_teams(self):
        mock_github.create_fake_org(teams=[mock_github.MockTeam('team1', ['bat', 'baz'])])
        git_hub = git.GitHub('foo', 'bar', 'bat')
        self.assertEquals(
            sorted(git_hub.expand_teams(['foo', 'OrgName/team1'])),
            sorted(['foo', 'bat', 'baz']))



from functools import partial

from github import UnknownObjectException

import mock_github
from base import MockPyGithubTests
from lgtm import pull_request_ready_to_merge as pull_request_ready_to_merge_


pull_request_ready_to_merge = partial(pull_request_ready_to_merge_, 'foo', 'bar', 'bat')


class PullRequestReadyToMergeTests(MockPyGithubTests):

    def test_ready(self):
        mock_github.create_fake_pull_request(id=1)
        self.assertTrue(pull_request_ready_to_merge(pr_number=1))

    def test_404(self):
        mock_github.create_fake_pull_request(id=1)
        with self.assertRaises(UnknownObjectException):
            self.assertTrue(pull_request_ready_to_merge(pr_number=2))

    def test_empty_owners_file(self):
        mock_github.create_fake_repo(file_contents={'OWNERS': ''})
        mock_github.create_fake_pull_request(id=1, comments=[])
        self.assertTrue(pull_request_ready_to_merge(pr_number=1))

    def test_no_reviewers_for_these_files(self):
        mock_github.create_fake_repo(file_contents={'OWNERS': '@bar *.js'})
        mock_github.create_fake_pull_request(id=1, file_paths=['file1.txt'])
        self.assertTrue(pull_request_ready_to_merge(pr_number=1))

    def test_single_owner_is_pr_author(self):
        mock_github.create_fake_repo(file_contents={'OWNERS': '@bar'})
        mock_github.create_fake_pull_request(author='bar', comments=[])
        self.assertTrue(pull_request_ready_to_merge(pr_number=1))

    def test_owner_with_no_at_symbol(self):
        mock_github.create_fake_repo(file_contents={'OWNERS': 'bar'})
        mock_github.create_fake_pull_request(author='bar', comments=[])
        self.assertTrue(pull_request_ready_to_merge(pr_number=1))



from base import MockPyGithubTests
from lgtm.integrations import jenkins


class JenkinsIntegrationTests(MockPyGithubTests):

    def test_get_pull_request_dict(self):
        self.assertEquals(
            jenkins.get_pull_request_dict({
                'ghprbPullLink': 'https://github.com/OrgName/repo-name/pull/387',
            }),
            dict(github_org='OrgName', github_repo='repo-name', github_pr_number='387'))

    def test_get_pull_request_dict_rerun(self):
        self.assertEquals(
            jenkins.get_pull_request_dict({
                'GIT_URL': 'git@github.com:OrgName/repo-name.git',
                'GIT_BRANCH': 'origin/pr/387/merge',
            }),
            dict(github_org='OrgName', github_repo='repo-name', github_pr_number='387'))

    def test_parse_pull_link(self):
        self.assertEquals(
            jenkins.parse_pull_link('https://github.com/OrgName/repo-name/pull/1'),
            dict(github_org='OrgName', github_repo='repo-name', github_pr_number='1'))

    def test_parse_pull_link_bad(self):
        self.assertEquals(jenkins.parse_pull_link('foobar'), None)

    def test_parse_git_url(self):
        self.assertEquals(
            jenkins.parse_git_url('git@github.com:OrgName/repo-name.git'),
            dict(github_org='OrgName', github_repo='repo-name'))

    def test_parse_git_url_bar(self):
        self.assertEquals(jenkins.parse_git_url('foobar'), None)

    def test_parse_git_branch(self):
        self.assertEquals(
            jenkins.parse_git_branch('origin/pr/387/merge'),
            dict(github_pr_number='387'))

    def test_parse_git_branch_bad(self):
        self.assertEquals(
            jenkins.parse_git_branch('foobar'),
            None)



"""
Command line interface. Parses sys.argv arguments, imports environment variables, executes
pull_request_ready_to_merge() with the supplied parameters, prints any logging output to stdout, and
exits with an appropriate exit code.
"""
import argparse
import logging
import os
import sys
import pkg_resources

from lgtm import integrations
from lgtm import pull_request_ready_to_merge


logger = logging.getLogger(__name__)


def get_options_parser(args=None, do_exit=True):
    """
    Parses and validates sys.argv + environment variables into an options object
    :return: the options object
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--github-token',
        help='GitHub API Token, can also use GITHUB_TOKEN environment variable',
        default=os.environ.get('GITHUB_TOKEN'))
    parser.add_argument('--github-org', help='GitHub organization name')
    parser.add_argument('--github-repo', help='Pull request repository name')
    parser.add_argument('--github-pr-number', help='Pull request number')
    parser.add_argument('--owners-file', help='Relative path to OWNERS file', default='OWNERS')
    parser.add_argument(
        '--integration',
        help='Extract org/repo/pr from environment variables specific to a platform',
        choices=['jenkins', ],
        default=None)
    parser.add_argument('--version', help='Print version and exit', action='store_true')
    parser.add_argument(
        '--verbose',
        help='Print commands that are running and other debug info',
        action='store_true')
    options = parser.parse_args(args)
    logging.basicConfig(format='%(message)s')
    logger.setLevel(logging.DEBUG if options.verbose else logging.INFO)
    if options.integration:
        defaults = integrations.get_options_defaults_dict(options.integration)
        for option, value in defaults.items():
            setattr(options, option, value)
        logger.debug('Options: %r' % options)
    if options.version:
        return options
    required = ['github_token', 'github_org', 'github_repo', 'github_pr_number']
    if not all([getattr(options, option) for option in required]):
        parser.print_usage()
        if do_exit:
            exit()
    options.github_pr_number = int(options.github_pr_number)
    return options


def main(args=None, do_exit=True):
    """
    The main body of the lgtm command line tool
    :return: Zero if the PR is ready to merge, one if it's not
    """
    options = get_options_parser(args, do_exit=do_exit)
    if options.version:
        logger.info(pkg_resources.require('lgtm')[0].version)
        return 0
    ready_to_merge = pull_request_ready_to_merge(
        github_token=options.github_token,
        org=options.github_org,
        repo=options.github_repo,
        pr_number=options.github_pr_number,
        owners_file=options.owners_file,
    )
    if ready_to_merge:
        logger.info('Pull request is ready to merge.')
        return 0
    logger.info('Pull Request is NOT ready to merge.')
    return 1


if __name__ == '__main__':
    sys.exit(main())



import logging

from dateutil import parser as dateutil_parser
from github import Github as PyGithub
from github import UnknownObjectException
import utils


logger = logging.getLogger(__name__)


# prefix to the notification comment, also used to identify the comment during future runs
REVIEW_COMMENT_PREFIX = 'This pull request requires a code review.'
# a list of patterns to match against comments that indicate a successful code review
LGTM_ALIASES = [
    'lgtm',
    ':shipit:',
]


class GitHub(object):
    """
    Wrapper around PyGithub with helpers for getting repo owners, a handle to pull request object.
    """

    def __init__(self, github_token, org_name, repo_name):
        self._git_api = PyGithub(github_token)
        self.org_name = org_name
        self.repo_name = repo_name
        self.org = None
        self.repo = None
        self.current_user_login = None

    def _connect(self):
        self.org = self._git_api.get_organization(self.org_name)
        self.repo = self.org.get_repo(self.repo_name)
        self.current_user_login = self._git_api.get_user().name

    def read_file_lines(self, file_path='OWNERS'):
        """
        Get a list of strings, one per line in the file
        :param owners_file: A relative path to the file
        :return: a list of line strings
        """
        self._connect()
        try:
            owner_file_contents = self.repo.get_file_contents(file_path)
            return owner_file_contents.decoded_content.split('\n')
        except UnknownObjectException:
            return []

    def get_pull_request(self, pr_number):
        """
        :param pr_number: A GitHub pull request ID
        :return: Handle to PullRequest helper
        """
        self._connect()
        return PullRequest(self, pr_number)

    def get_team_members(self, team_name):
        """
        Returns a list of GitHub user names for the members of a GitHub team
        :param team_name: GitHub team name, like 'OrgName/team1'
        :return: a list of GitHub user names
        """
        self._connect()
        assert '/' in team_name
        org, team_name = team_name.split('/')  # ex: NerdWallet/dit
        teams = self.org.get_teams()
        teams = [t for t in teams if t.name == team_name]
        if not teams:
            return []
        return [m.login for m in teams[0].get_members()]

    def expand_teams(self, logins_and_teams_list, except_login=None):
        """
        Given a list of GitHub user names and team names, return a set of user names with the team
        members expanded.
        :param logins_and_teams_list: list of GitHub user names and team names
        :return: list of GitHub user names
        """
        self._connect()
        logins = list()
        for login_or_team in logins_and_teams_list:
            if login_or_team == except_login:
                continue
            if '/' in login_or_team:
                for login in self.get_team_members(login_or_team):
                    logins.append(login)
            else:
                logins.append(login_or_team)
        return utils.ordered_set(logins)


class PullRequest(object):
    """
    A helper object for GitHub pull requests that can pull reviews based on an OWNERS file,
    pull comments and author. It can also notify users on the pull request via a comment, and
    determine whether a pull request has been signed off on by the required reviewers.
    """

    def __init__(self, git_hub, pr_number):
        self._git_hub = git_hub
        self.pr_number = pr_number
        self._pr = git_hub.repo.get_pull(self.pr_number)

    @property
    def files(self):
        return [f.filename for f in self._pr.get_files()]

    @property
    def comments(self):
        """
        Gets the comments on a pull request.
        :return: a list of (user name, comment text) tuples
        """
        comments = self._pr.get_issue_comments()
        return comments

    @property
    def last_commit_date(self):
        """
        Gets the date of the most recent commit on a pull request.
        :return: a datetime object
        """
        commits = self._pr.get_commits()
        commit_date_strings = [c.last_modified for c in commits]
        commit_dates = [dateutil_parser.parse(d).replace(tzinfo=None) for d in commit_date_strings]
        return max(commit_dates) if commit_dates else None

    @property
    def author(self):
        """
        Get the GitHub user name of the pull request author
        :return: a GitHub user name
        """
        return self._pr.user.login

    def assign_to(self, login):
        """
        Assign a PR to a specific user.

        :param login: A list of GitHub user name
        """
        issue = self._git_hub.repo.get_issue(self.pr_number)
        issue.edit(assignee=login)

    def _get_existing_comment(self):
        for comment in self.comments:
            author = comment.user.login
            if author != self._git_hub.current_user_login:
                continue
            if comment.body.startswith(REVIEW_COMMENT_PREFIX):
                return comment
        return None

    def _make_mention_string(self, logins):
        return ' '.join(['@%s' % login for login in logins])

    def generate_comment(self, reviewers, required, prefix=None):
        if not reviewers and not required:
            return None
        prefix = prefix or REVIEW_COMMENT_PREFIX
        message = '%s\n\n' % prefix
        if required:
            message += 'All of the following reviewers must sign off: '
            message += self._make_mention_string(required)
            message += '\n\n'
            message += 'Optional: %s' % self._make_mention_string(reviewers)
        else:
            message += 'One of the following reviewers must sign off: '
            message += self._make_mention_string(reviewers)
        message = message.strip()
        return message

    def create_or_update_comment(self, message):
        """
        Notify a list of GitHub user names that they should review this pull request. Only notifies
        the users once.

        :param reviewers: A list of GitHub user names that should review the PR
        :param required: A list of GitHub user names that are required to review the PR
        :param prefix: A prefix to append to any automated comments
        """
        # see: https://github.com/blog/2178-multiple-assignees-on-issues-and-pull-requests
        # see: https://github.com/PyGithub/PyGithub/issues/404
        if not message:
            return
        existing_comment = self._get_existing_comment()
        if existing_comment:
            if existing_comment.body == message:
                return
            existing_comment.edit(message)
        else:
            self._pr.create_issue_comment(message)

    def one_has_signed_off(self, reviewers):
        if not reviewers:
            return True
        sign_offs = self.signed_off_by()
        for reviewer in reviewers:
            assert '/' not in reviewer, 'Teams of reviewers should be expanded already'
            if reviewer in sign_offs:
                return True
        return False

    def all_have_signed_off(self, required_reviewers):
        if not required_reviewers:
            return True
        sign_offs = self.signed_off_by()
        sign_offs = self._git_hub.expand_teams(sign_offs, except_login=self.author)
        for required_reviewer in required_reviewers:
            if required_reviewer not in sign_offs:
                return False
        return True

    def signed_off_by(self, except_login=None):
        """
        Get the list of GitHub user names who have signed off on the pull request.
        :param except_login: A GitHub user name who should not be allowed to sign off
        :return: A list of GitHub user names
        """
        except_login = except_login or self.author
        lgtm_logins = list()
        last_commit_date = self.last_commit_date
        for comment in self.comments:
            author = comment.user.login
            # do not let the author sign off on their own PR
            if author == except_login:
                continue
            # ignore any lgtm comments prior to the most recent commit (need to lgtm again)
            if last_commit_date and comment.created_at < last_commit_date:
                continue
            comment_body = comment.body.lower()
            for lgtm_token in LGTM_ALIASES:
                # TODO: regex
                if lgtm_token in comment_body:
                    lgtm_logins.append(author)
        return utils.ordered_set(lgtm_logins)



from collections import OrderedDict


def ordered_set(item_list):
    """
    Return a list of the unique set of items, with order maintained by first appearance
    :param item_list: A list of items that can be compared to each other
    :return: a list of unique items
    """
    # useful b/c Python does not have an ordered set builtin
    items = OrderedDict()
    for item in item_list:
        items[item] = item
    return items.values()



import mock
import unittest

import mock_github


class MockPyGithubTests(unittest.TestCase):

    def setUp(self):
        github_patcher = mock.patch('lgtm.git.PyGithub', mock_github.MockPyGithub)
        github_patcher.start()
        self.addCleanup(github_patcher.stop)
        self.org = mock_github.create_fake_org()
        self.repo = mock_github.create_fake_repo()
        super(MockPyGithubTests, self).setUp()



from base import MockPyGithubTests
from lgtm.integrations import base


class IntegrationBaseTests(MockPyGithubTests):

    def test_get_options_defaults_dict_jenkins(self):
        self.assertEquals(base.get_options_defaults_dict('jenkins'), {})

    def test_get_options_defaults_dict_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            base.get_options_defaults_dict('foobar')



def get_options_defaults_dict(integration_type):
    if integration_type == 'jenkins':
        import jenkins
        return jenkins.get_pull_request_dict()
    else:
        raise NotImplementedError()



import unittest

from lgtm import owners


class GetOwnersTests(unittest.TestCase):

    def test_get_owners_of_files(self):
        reviewers, required = owners.get_owners_of_files([('foo', None)], files=['bar', 'bat'])
        self.assertEquals(reviewers, ['foo'])
        self.assertEquals(required, [])

    def test_get_owners_of_files_empty(self):
        reviewers, required = owners.get_owners_of_files([], files=['bar', 'bat'])
        self.assertEquals(reviewers, [])
        self.assertEquals(required, [])

    def test_get_owners_of_files_glob(self):
        reviewers, required = owners.get_owners_of_files([('foo', '*.js')], files=['bar', 'bat.js'])
        self.assertEquals(reviewers, ['foo'])
        self.assertEquals(required, ['foo'])

    def test_get_owners_of_files_glob_subdir(self):
        reviewers, required = owners.get_owners_of_files(
            [('foo', '*/bar/*')], files=['foo/bar/bat'])
        self.assertEquals(reviewers, ['foo'])
        self.assertEquals(required, ['foo'])

    def test_get_owners_of_files_glob_all(self):
        reviewers, required = owners.get_owners_of_files([('foo', '*')], files=['bar', 'bat'])
        self.assertEquals(reviewers, ['foo'])
        self.assertEquals(required, ['foo'])

    def get_owners_of_files_order(self):
        reviewers, required = owners.get_owners_of_files([
                ('foo', None),
                ('bar', '*'),
                ('bat', '*'),
                ('foo', '*'),
                ('foo', '*.js'),
            ], files=['bar', 'bat'])
        self.assertEquals(reviewers, ['foo', 'bar', 'bat'])
        self.assertEquals(required, ['foo'])


class OwnersParseTests(unittest.TestCase):

    def test_parse(self):
        id_glob_tuples = owners.parse([
            '@github-user',
            '@OrgName/team-name',
            '@github-user2 *.js',
            '@github-user3 */subdir/*',
        ])
        self.assertEquals(id_glob_tuples, [
            ('github-user', None),
            ('OrgName/team-name', None),
            ('github-user2', '*.js'),
            ('github-user3', '*/subdir/*'),
        ])



from dateutil import parser as dateutil_parser
from github import UnknownObjectException


_org_state = {
    'name': None,
    'current_user_login': None,
    'teams': [],
}


def create_fake_org(**kwargs):
    defaults = dict(
        org_name='OrgName',
        current_user_login='bot',
        teams=[
            MockTeam('team1', ['bat', 'baz']),
        ],
    )
    defaults.update(kwargs)
    global _org_state
    _org_state.update(defaults)


_repo_state = {
    'name': None,
    'file_contents': {},
}


def create_fake_repo(**kwargs):
    defaults = dict(
        name='repo-name',
        file_contents={
            'OWNERS': '@foo\n'
                      '@bar *.js\n'
                      '@OrgName/team1 build/*\n'
            },
    )
    defaults.update(kwargs)
    global _repo_state
    _repo_state.update(defaults)


_pull_request_state = {}


def create_fake_pull_request(**kwargs):
    defaults = dict(
        id=1,
        author='bat',
        last_commit_date='2016-01-01 00:00:01',
        file_paths=[
            'file1',
            'file2.js'
            'build/foo.txt',
        ],
        comments=[
            ('2016-01-01 00:00:00', 'foo', 'lgtm'),  # too early
            ('2016-01-01 00:00:02', 'bat', 'lgtm'),  # from the author
            ('2016-01-01 00:00:03', 'boo', 'lgtm'),  # not from an owner
            ('2016-01-01 00:00:04', 'baz', 'some comment'),  # not an lgtm comment
            ('2016-01-01 00:00:05', 'foo', 'lgtm'),  # this one is a successful lgtm
        ],
    )
    defaults.update(kwargs)
    global _pull_request_state
    _pull_request_state = {defaults.get('id'): defaults}


class MockPyGithub(object):

    def __init__(self, login_or_token):
        self.login_or_token = login_or_token

    def get_organization(self, login):
        # TODO: check that this org name is correct
        return MockOrganization(login)

    def get_user(self, login=None):
        login = login or _org_state['current_user_login']
        return MockUser(login)


class MockUser(object):

    def __init__(self, login):
        self.login = login
        self.name = login


class MockTeam(object):

    def __init__(self, name, member_logins=None):
        self.name = name
        self.member_logins = member_logins or []

    def get_members(self):
        return [MockUser(login) for login in self.member_logins]


class MockOrganization(object):

    def __init__(self, login):
        self.login = login

    def get_repo(self, full_name_or_id, lazy=True):
        return MockRepository(full_name_or_id)

    def get_teams(self):
        return _org_state['teams']


class MockRepository(object):

    def __init__(self, full_name_or_id):
        self.full_name_or_id = full_name_or_id

    def get_pull(self, number):
        if number not in _pull_request_state:
            raise UnknownObjectException(404, 'not found')
        return MockPullRequest.from_state(id=number)

    def get_issue(self, number):
        if number not in _pull_request_state:
            raise UnknownObjectException(404, 'not found')
        return MockIssue()

    def get_file_contents(self, path, ref=None):
        if path not in _repo_state['file_contents']:
            raise UnknownObjectException(404, 'not found')
        return MockFileContents(_repo_state['file_contents'].get(path))


class MockFileContents(object):

    def __init__(self, contents):
        self.decoded_content = contents


class MockPullRequest(object):

    def __init__(self, id, author, last_commit_date, file_paths, comments, **kwargs):
        self._id = id
        self._author = author
        self._last_commit_date_str = last_commit_date
        self._last_commit_date = dateutil_parser.parse(last_commit_date)
        self._file_paths = file_paths
        self._comments = comments

    @classmethod
    def from_state(cls, id):
        return cls(**_pull_request_state.get(id))

    @property
    def user(self):
        return MockUser(self._author)

    def get_issue_comments(self):
        return [MockComment(*comment) for comment in self._comments]

    def create_issue_comment(self, body):
        pass

    def get_files(self):
        return [MockFile(f) for f in self._file_paths]

    def get_commits(self):
        # don't need anything except the most recent date
        return [MockCommit(self._last_commit_date_str), ]


class MockCommit(object):

    def __init__(self, last_modified):
        self.last_modified = last_modified


class MockFile(object):

    def __init__(self, filename):
        self.filename = filename


class MockComment(object):

    def __init__(self, created_at, login, body):
        self.created_at = dateutil_parser.parse(created_at)
        self.user = MockUser(login)
        self.body = body


class MockIssue(object):

    def __init__(self):
        self.assignee = None

    def edit(self, assignee):
        self.assignee = assignee



"""
See: https://www.chromium.org/developers/owners-files

The syntax of the OWNERS file is, roughly:
lines     := (\s* line? \s* "\n")*
line      := directive
          | "per-file" \s+ glob \s* "=" \s* directive
          | comment
directive := "set noparent"
          |  "file:" glob
          |  email_address
          |  "*"
glob      := [a-zA-Z0-9_-*?]+
comment   := "#" [^"\n"]*
"""
import fnmatch
import logging
import utils


logger = logging.getLogger(__name__)


def parse(owners_lines):
    """
    takes a list of lines from a OWNERS text file and returns a list of
    :param owners_lines: a list of strings, one for each line of a OWNERS file
    :return: list of (ID, glob) tuples, where glob can be None
    """
    results = []
    for owner_line in owners_lines:
        # strip the leading @ for github users/teams
        if owner_line.startswith('@'):
            owner_line = owner_line[1:]
        owner_line = owner_line.strip()
        if not owner_line:
            continue
        if owner_line.startswith('#'):
            continue
        if ' ' in owner_line:
            result = tuple(owner_line.split(' ', 1))
        else:
            result = (owner_line, None)
        results.append(result)
    return results


def get_owners_of_files(owner_glob_tuple_list, files):
    """
    Given a list of (ID, glob) tuples and a list of files, return the set of IDs of reviewers
    :param owner_glob_tuple_list: a list of (ID, glob) tuples from OWNERS
    :param files: a list of files changed by the pull request
    :return: the list of IDs of reviewers who should review the PR, and a list of IDs that MUST
        sign off on a PR before it can be merged
    """
    # no ordered set in Python, use ordered dict w/ throw-away key
    # preserve order of input b/c we want to assign PR to first owner
    reviewers, required = list(), list()
    for owner, glob in owner_glob_tuple_list:
        if glob:
            matched_files = fnmatch.filter(files, glob)
            if matched_files:
                logger.debug('%s matches %r' % (owner, matched_files))
                reviewers.append(owner)
                required.append(owner)
        else:
            logger.debug('%s matches anything' % owner)
            reviewers.append(owner)
    return utils.ordered_set(reviewers), utils.ordered_set(required)



import mock_github
from base import MockPyGithubTests
from lgtm import console


class ConsoleTests(MockPyGithubTests):

    def test_get_options_parser(self):
        options = console.get_options_parser([
            '--github-token',
            'foo',
            '--github-org',
            'OrgName',
            '--github-repo',
            'repo-name',
            '--github-pr-number',
            '1',
        ], do_exit=False)
        self.assertEquals(options.github_token, 'foo')
        self.assertEquals(options.github_org, 'OrgName')
        self.assertEquals(options.github_repo, 'repo-name')
        self.assertEquals(options.github_pr_number, 1)

    def test_main_ready(self):
        mock_github.create_fake_pull_request(id=1)
        console.main([
            '--github-token',
            'foo',
            '--github-org',
            'OrgName',
            '--github-repo',
            'repo-name',
            '--github-pr-number',
            '1',
        ], do_exit=False)

    def test_main_not_ready(self):
        mock_github.create_fake_repo(name='repo-name', file_contents={
            'OWNERS': 'some-other-person'
            })
        mock_github.create_fake_pull_request(id=1)
        console.main([
            '--github-token',
            'foo',
            '--github-org',
            'OrgName',
            '--github-repo',
            'repo-name',
            '--github-pr-number',
            '1',
        ], do_exit=False)

    def test_main_version(self):
        console.main([
            '--version',
        ], do_exit=False)



from setuptools import setup, find_packages

install_requires = [
    'python-dateutil>=2.4.2,<3.0.0',
    'PyGithub>=1.26.0,<2.0.0',
]

setup(
    name='lgtm',
    version='0.0.8',
    packages=find_packages(exclude=['tests', 'lgtm/tests']),
    install_requires=install_requires,
    include_package_data=True,
    author='Chase Seibert',
    author_email='cseibert@nerdwallet.com',
    license='Other/Proprietary License',
    description='A pull request approval system using GitHub protected branches and OWNERS files.',
    long_description='',
    url='https://github.com/nerdwallet/github-lgtm',
    entry_points={
        'console_scripts': ['lgtm=lgtm.console:main'],
    },
)



import os
import re


def _match(regex, input_str):
    match = re.match(regex, input_str)
    if match:
        return match.groupdict()
    return None


def parse_pull_link(pr_link):
    # ex: https://github.com/OrgName/repo-name/pull/1
    return _match(
        r'https?://github.com/(?P<github_org>[^/]+)/(?P<github_repo>[^/]+)/'
        r'pull/(?P<github_pr_number>\d+)',
        pr_link)


def parse_git_url(git_url):
    # ex: git@github.com:OrgName/repo-name.git
    return _match(r'git@github.com:(?P<github_org>[^/]+)/(?P<github_repo>[^/]+).git', git_url)


def parse_git_branch(git_branch):
    # ex: origin/pr/387/merge
    return _match(r'[^/]+/pr/(?P<github_pr_number>\d+)', git_branch)


def get_pull_request_dict(env=None):
    env = env or os.environ
    pull_request_dict = {}
    for env_var_name, parser in (
        ('ghprbPullLink', parse_pull_link),
        ('GIT_URL', parse_git_url),
        ('GIT_BRANCH', parse_git_branch),
    ):
        env_var_value = env.get(env_var_name)
        if env_var_value:
            info = parser(env_var_value)
            if info:
                pull_request_dict.update(info)
    return pull_request_dict
